/* Weather Display with PushButton Scene Activator
**************************************************
******Written by Jim BulldogLowell@gmail.com******
**************************************************
This sketch works with MySensors and your Vera and creates two devices, a 
thermometer and hygrometer which will report temperature and RH back to Vera.

In addition to that, it contains the following capabilities:
a) basic thermometer/hygrometer updated on changes
b) regularly retrieves weather information in text or numbers from your vera
c) displays indoor and outdoor temperature and humidity
d) displays weather including current conditions and Hi's and Low's
e) allows the use of scenes in Vera to control backlit LCD display by reading
   the state of Variable2 of the hygrometer and turning it on if '1"
f) pushbutton interrupt displays and triggers alert message and changes variable in Vera
   enabling the device to trigger actions within Vera on the change of state
   of Variable3 of the hygrometer
g) nearly zero delays allows the interrupt to feedback very quickly, though you 
   may see delayed response then the unit is displaying indoor conditions.  
   This is due to the RF calls beinging placed in this part of the sketch.
   
-Connect your pusbutton to interrupt 1 (pin3) and use hardware debouncing (simple RC is enough).
-Connect your Radio and thermometer/hygrometer as outlined in the wwwMySensors.org webpage

I used WeatherUnderground and populated the variable fields with a scene that runs
every 10 mins.
Thanks to Hek and the rest of the MySensors team for enabling this!
*/ 
#include <Sleep_n0m1.h>
#include <SPI.h>
#include <EEPROM.h>  
#include <RF24.h>
#include <Sensor.h>  
#include <DHT.h> 
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
//
LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x20 for a 16 chars and 2 line display
//
#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define HUMIDITY_SENSOR_DIGITAL_PIN 4
unsigned long SLEEP_TIME = 5; // Sleep time between reads (in seconds)
//
Sensor gw;
DHT dht;
Sleep sleep;
float lastTemp;
float lastHum;
float temperature;
float humidity;
float temp;
boolean metric = false;
int OutdoorHumidity;
int OutdoorTemp;
int TodayLow;
int TodayHigh;
String TodayWeather= "No Weather Reported"; //*****
int ledStatus = 1;// to toggle LCD backlight led
int counter = 0;
int MessageToggle = 0;
int pin = 3;
unsigned long ElapsedTime = 0;
unsigned long time = millis();
volatile int state = LOW;
//
void setup()  
{ 
  Serial.begin(115200);
  pinMode(pin, OUTPUT);//interrupt on pin 3
  attachInterrupt(1, PushButton, CHANGE);//interrupt pin 3 is interrupt 1
  gw.begin();
  dht.setup(HUMIDITY_SENSOR_DIGITAL_PIN); 
  gw.sendSketchInfo("LCD Multi", "1.0");
  gw.sendSensorPresentation(CHILD_ID_HUM, S_HUM);
  gw.sendSensorPresentation(CHILD_ID_TEMP, S_TEMP);
  gw.getStatus(CHILD_ID_HUM, V_VAR1);//current exterior humidity
  HumStatus(gw.getMessage());
  gw.getStatus(CHILD_ID_HUM, V_VAR2);//Backlight Toggle 0=off 1==ON
  ledToggle(gw.getMessage());
  gw.getStatus(CHILD_ID_HUM, V_VAR3);//PushButton toggle
  gw.getStatus(CHILD_ID_TEMP,V_VAR1);//current exterior temperature
  TempStatus(gw.getMessage());
  gw.getStatus(CHILD_ID_TEMP,V_VAR2);//today's high temperature
  LowStatus(gw.getMessage());
  gw.getStatus(CHILD_ID_TEMP,V_VAR3);//today's low temperature
  HighStatus(gw.getMessage());
  gw.getStatus(CHILD_ID_TEMP,V_VAR4);//todays weather
  WeatherStatus(gw.getMessage());
  temphumidity();
  //
  gw.sendVariable(CHILD_ID_HUM, V_VAR3, 0);//reset PushButton toggle
  //
  lcd.init(); // initialize the lcd 
  // Print a message to the LCD.
  lcd.backlight();
  //lcd.setBacklight(LOW);******** come back to this
  lcd.setCursor(0,0);
  lcd.print("Place Your");// You can put a custome initialization message to display
  lcd.setCursor(0,1);
  lcd.print("Initialization");
  delay(2000);
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Messages");
  lcd.setCursor(0,1);
  lcd.print("Here");
  delay(2000);
  time=millis();
  delay(10);
}
void loop()      
{ 
  ElapsedTime=millis()-time;
  // the following switch rotates the variables to be called back to the device.
  // calling all of the variables every turn of the loop is too much radio activity
  // and causes a big delay which does not perform well with the interrupt trigger
  switch (counter){ 
    case 0:
    gw.getStatus(CHILD_ID_HUM, V_VAR1);
    HumStatus(gw.getMessage());
    Serial.println("0: updated ext. humidity");
    break;
    case (5)://5min
    gw.getStatus(CHILD_ID_HUM, V_VAR2);
    ledToggle(gw.getMessage());
    Serial.println("5: updated backlight led status");
    if (ledStatus == 1) {//checking to toggle LCD's internal backlight LED
       lcd.backlight();
    }
    else{
       lcd.noBacklight();
    }
    break;
    case (10):
    gw.getStatus(CHILD_ID_TEMP, V_VAR1);
    TempStatus(gw.getMessage());
    Serial.println("10: updated ext temperature");
    break;
    case (15):
    gw.getStatus(CHILD_ID_TEMP, V_VAR2);
    LowStatus(gw.getMessage());
    Serial.println("15: updated ext low");
    //
    break;
    case(20):
    gw.getStatus(CHILD_ID_TEMP, V_VAR3);
    HighStatus(gw.getMessage());
    Serial.println("20: updated ext high");
    break;
    case (25):
    gw.getStatus(CHILD_ID_TEMP, V_VAR4);
    WeatherStatus(gw.getMessage());
    Serial.print("25: updated ext weather: ");
    Serial.println(TodayWeather);
    break;
    case (30):
    temphumidity();
    break;
    //
    ElapsedTime = millis()-time;
  }
  while (ElapsedTime<3000){
    // lcd print outdoor temp and humidity
    updateOutsideTemp();
    if (MessageToggle == 1){  // each while loop looks for the interrupt flag
      AirConMessage();
    }
    ElapsedTime = millis()-time;
  }
  while (ElapsedTime<6000){
    // print today's high and low temperature
    updateHighLow();
    if (MessageToggle == 1){
      AirConMessage(); // I set this up for an air conditioner scene, sorry for the wierd lexicon
    }
    ElapsedTime=millis()-time;
  }
  while(ElapsedTime<9000){
    // print today's weather forecast
    updateForecast();
    if (MessageToggle == 1){
      AirConMessage();
    }
    ElapsedTime=millis()-time;
  }
  while (ElapsedTime<12000){
    // print indoor temp and humidity
    updateIndoor();
    if (MessageToggle == 1){
      AirConMessage();
    }
    ElapsedTime=millis()-time;
  }
  time=millis();  // reset the timer
  counter=counter++;  //advance the counter
  Serial.println (counter);
  if (counter > 29){
  counter=0;
  }
}
//****** Print to LCD Functions
void updateIndoor(){
  lcd.setCursor(0,0);
  lcd.print("Temperature: ");
  lcd.print(round(temp));
  lcd.print("F ");
  lcd.setCursor(0,1);
  lcd.print("Humidity:    ");
  lcd.print(round(humidity));
  lcd.print("% ");
}
void updateHighLow(){
  lcd.setCursor(0,0);
  lcd.print("Today's Low: ");
  lcd.print(round(TodayLow));
  lcd.print("F ");
  lcd.setCursor(0,1);
  lcd.print("       High: ");
  lcd.print(round(TodayHigh));
  lcd.print("F ");
}
void updateOutsideTemp(){
  lcd.setCursor(0,0);
  lcd.print("Outside Temp:");
  lcd.print(OutdoorTemp);
  lcd.print("F ");
  lcd.setCursor(0,1);
  lcd.print("Humidity:    ");
  lcd.print(OutdoorHumidity);
  lcd.print("% ");
}
void updateForecast(){
  lcd.setCursor(0,0);
  lcd.print("Today's Weather ");
  lcd.setCursor(0,1);
  lcd.print(TodayWeather);
  lcd.print("               ");
}
void AirConMessage(){
  if (ledStatus == 1){
    lcd.noBacklight();
    for (int i=0; i < 10; i++){
    lcd.backlight();
    delay(100);
    lcd.noBacklight();
    delay(100);
    lcd.backlight();
    }
    lcd.setCursor(0,0);
    lcd.print("*Scene Actuator*");
    lcd.setCursor(0,1);
    lcd.print("***ACTIVATED****");
    delay(3000);
    MessageToggle = 0;
    gw.sendVariable(CHILD_ID_HUM, V_VAR3, 1);
  }
}
// Data Calls
void TempStatus(message_s message){
  if (message.header.type==V_VAR1) {
     OutdoorTemp = atoi(message.data);
   }
}
void LowStatus(message_s message){
  if (message.header.type==V_VAR2) {
     TodayLow = atoi(message.data);
   }
}
void HighStatus(message_s message){
  if (message.header.type==V_VAR3) {
     TodayHigh = atoi(message.data);
   }
}
void WeatherStatus(message_s message){
  if (message.header.type==V_VAR4) {
     String Weather = String(message.data);
     int locator = Weather.indexOf("@");
     Weather = Weather.substring(0, locator);
     TodayWeather= Weather;
   }
}
void ledToggle(message_s message){
  if (message.header.type==V_VAR2) {
    ledStatus = atoi (message.data);
   }
}
void HumStatus(message_s message){
  if (message.header.type==V_VAR1) {
     OutdoorHumidity = atoi(message.data);
   }
}
void temphumidity(){
  temperature = dht.getTemperature();
    temp = dht.toFahrenheit(temperature);
    if (isnan(temperature)) {
      Serial.println("Failed reading temperature from DHT");
    } else if (temperature != lastTemp) {
      lastTemp = temperature;
    if (!metric) {
      temperature = dht.toFahrenheit(temperature);
    }
    gw.sendVariable(CHILD_ID_TEMP, V_TEMP, temperature, 1);
      Serial.print("T: ");
      Serial.println(temperature);
    }
    humidity = dht.getHumidity();
    if (isnan(humidity)) {
      Serial.println("Failed reading humidity from DHT");
    } 
    else if (humidity != lastHum) {
      lastHum = humidity;
      gw.sendVariable(CHILD_ID_HUM, V_HUM, humidity, 1);
      Serial.print("H: ");
      Serial.println(humidity);
    }temperature = dht.getTemperature();
    temp = dht.toFahrenheit(temperature);
    if (isnan(temperature)) {
      Serial.println("Failed reading temperature from DHT");
    } else if (temperature != lastTemp) {
      lastTemp = temperature;
    if (!metric) {
      temperature = dht.toFahrenheit(temperature);
    }
    gw.sendVariable(CHILD_ID_TEMP, V_TEMP, temperature, 1);
      Serial.print("T: ");
      Serial.println(temperature);
    }
    humidity = dht.getHumidity();
    if (isnan(humidity)) {
      Serial.println("Failed reading humidity from DHT");
    } 
    else if (humidity != lastHum) {
      lastHum = humidity;
      gw.sendVariable(CHILD_ID_HUM, V_HUM, humidity, 1);
      Serial.print("H: ");
      Serial.println(humidity);
      Serial.println(":30: updated internal temp and humidity");
    }
}
void PushButton(){ //interrupt.
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  // If interrupts come faster than 200ms, assume it's a bounce and ignore
  if (interrupt_time - last_interrupt_time > 200) 
  {
  MessageToggle = 1;
  Serial.println("INTERRUPT");
  }
  last_interrupt_time = interrupt_time;
}

